#!/usr/bin/python3
#
#  create_project.py
#
#  Created by w0rng on 12.09.2018.
#  Copyright © 2020 w0rng. All rights reserved.
#


import json
import os
import logging


def create_project(project_type, project_name):
    path_configs = os.path.expanduser("~") + '/.config/project_creator/'

    if project_type not in os.listdir(path_configs):
        logging.error(f'Конфиг {project_type} не найден')
        exit(1)

    if os.path.exists(project_name):
        logging.error(f'Проект {project_name} уже существует')
        exit(1)
    else:
        os.mkdir(project_name)
        os.chdir(f'./{project_name}/')

    with open(path_configs+project_type) as f:
        config = json.load(f)

    if 'path' in config:
        logging.info('Создание папок')
        create_folders(config['path'])

    if 'files' in config:
        logging.info('Создание файлов')
        for f in config['files']:
            create_file(f, config['files'][f])

    if 'commands' in config:
        logging.info('Выполнение команд')
        for commands in config['commands']:
            run_command(commands)


def __get_folders(folders, path=''):
    """Возвращает пути к папкам"""
    for folder in folders:
        yield f'{path}{folder}/'
        if len(folders[folder]):
            yield from __get_folders(folders[folder], f'{path}{folder}/')


def create_folders(folders):
    """Создает структуру папок"""
    for path in __get_folders(folders):
        if not os.path.exists(path):
            os.mkdir(path)


def create_file(name, lines):
    """Создает файл name с содержимым lines"""
    if os.path.exists(name):
        logging.warning(f'{name} существует')
        return None
    with open(name, 'w') as file:
        for line in lines:
            file.write(line)


def run_command(commands):
    """Выполняет команду commands"""
    os.system(commands)



#
#  list_projects.py
#
#  Created by w0rng on 28.12.2020.
#  Copyright © 2020 w0rng. All rights reserved.
#


import os


def print_list_projects():
    path_configs = os.path.expanduser('~') + '/.config/project_creator/'
    for i in os.listdir(path_configs):
        print(i)



#
#  __main__.py
#
#  Created by w0rng on 28.12.2020.
#  Copyright © 2020 w0rng. All rights reserved.
#


import sys
import logging
from .create_project import create_project
from .list_projects import print_list_projects
from .create_config import create_config
from .remove_config import remove_config


logging.basicConfig(
    level=logging.ERROR,
    format='%(name)s - %(levelname)s - %(message)s'
)


def check_create_command():
    if len(sys.argv) < 3:
        logging.error('Не введен тип проекта')
        exit(1)
    elif len(sys.argv) < 4:
        logging.error('Не введено название проекта')
        exit(1)

    create_project(sys.argv[2], sys.argv[3])


def check_create_config():
    if len(sys.argv) < 3:
        logging.error('Не введено название нового конфига')
        exit(1)
    elif len(sys.argv) < 4:
        logging.error('Не введено название папки')
        exit(1)
    create_config(sys.argv[2], sys.argv[3])


def check_remove_config():
    if len(sys.argv) < 3:
        logging.error('Не введено название нового конфига')
        exit(1)
    remove_config(sys.argv[2])


def print_help_info():
    print('Введите pjc create {config_name} {name}',
          'чтобы создать проект')
    print('Введите pjc list чтобы посмотреть список доступных конфигов')
    print('Введите pjc newconfig {config_name} {path}',
          'чтобы сгенерировать конфиг')
    print('Введите pjc remove {config_name} чтобы удалить конфиг')


def main():
    commands = {
        'create': check_create_command,
        'list': print_list_projects,
        'newconfig': check_create_config,
        'remove': check_remove_config
    }

    if len(sys.argv) < 2:
        print_help_info()
        exit(0)

    if (command := sys.argv[1]) in commands:
        commands[command]()
    else:
        logging.warning('Команда не найдена')


if __name__ == "__main__":
    main()



#
#  create_config.py
#
#  Created by w0rng on 28.12.2020.
#  Copyright © 2020 w0rng. All rights reserved.
#

import os
import logging
from .config import IGNORE_PATHS
import json


def create_config(config_name, path):
    path_configs = os.path.expanduser("~") + '/.config/project_creator/'

    if config_name in os.listdir(path_configs):
        logging.error(f'{config_name} уже существует')
        exit(1)

    if not os.path.exists(path):
        logging.error(f'Папка {config_name} не найдена')
        exit(1)

    config = {
        'path': get_paths_config(path),
        'files': get_files_config(path)
    }

    with open(f'{path_configs}/{config_name}', 'w') as outfile:
        json.dump(config, outfile, indent=4)


def get_paths_config(path):
    logging.info('Генерация списка папок')
    result = {}
    for item in __get_paths_structure(path):
        p = result
        for x in item.split('/'):
            p = p.setdefault(x, {})
    return result[path]


def get_files_config(path):
    logging.info('Генерация списка файлов')
    result = {}
    for i in __get_files_structure(path):
        i = '/'.join(i.split('/')[1:])
        result[i] = open(f'{path}/{i}', 'r').readlines()
    return result


def __get_paths_structure(path):
    for i in os.listdir(path):
        if not os.path.isdir(f'{path}/{i}'):
            logging.debug(f'Файл {i} пропущен при генерации структуры папок')
            continue
        if i in IGNORE_PATHS:
            logging.debug(f'Папка {i} проигнорированна')
            continue

        yield f'{path}/{i}'
        yield from __get_paths_structure(f'{path}/{i}')


def __get_files_structure(path):
    for i in os.listdir(path):
        if not os.path.isdir(f'{path}/{i}'):
            yield f'{path}/{i}'
            continue
        if i in IGNORE_PATHS:
            logging.debug(f'Папка {i} проигнорированна')
            continue

        yield from __get_files_structure(f'{path}/{i}')



#
#  remove_config.py
#
#  Created by w0rng on 28.12.2020.
#  Copyright © 2020 w0rng. All rights reserved.
#

import os


def remove_config(config_name):
    path_configs = os.path.expanduser("~") + '/.config/project_creator/'

    if config_name not in os.listdir(path_configs):
        logging.error(f'Конфиг {config_name} не найден')
        exit(1)

    os.remove(f'{path_configs}{config_name}')



#
#  __init__.py
#
#  Created by w0rng on 23.12.2020.
#  Copyright © 2020 w0rng. All rights reserved.
#

import os


__version__ = '0.2.5.1'
__author__ = 'w0rng'


user_dir = os.path.expanduser("~")

if not os.path.exists(f'{user_dir}/.config/'):
    os.mkdir(f'{user_dir}/.config/')

if not os.path.exists(f'{user_dir}/.config/project_creator/'):
    os.mkdir(f'{user_dir}/.config/project_creator/')



from setuptools import setup, find_packages
import projectcreator
from os.path import join, dirname


setup(
    name='pjc',
    version=projectcreator.__version__,
    author=projectcreator.__author__,
    description='Package for create projects',
    long_description=open(join(dirname(__file__), 'README.rst')).read(),
    url='https://github.com/w0rng/project-creator',
    packages=find_packages(),
    entry_points={
        'console_scripts':
            ['pjc = projectcreator.main:main']
    }
)
